#!/usr/bin/env python
from setuptools import setup
setup(
  name = 'cs.x',
  author = 'Cameron Simpson',
  author_email = 'cs@cskk.id.au',
  version = '20211208',
  url = 'https://bitbucket.org/cameron_simpson/css/commits/all',
  description =
    'X(), for low level debugging.',
  long_description =
    ('X(), for low level debugging.\n'    
 '\n'    
 '*Latest release 20211208*:\n'    
 'X_via_tty: now accept full path to a tty in $CS_X_VIA_TTY to aid messaging '    
 'to other terminals.\n'    
 '\n'    
 'X() is my function for low level ad hoc debug messages.\n'    
 'It takes a message and optional format arguments for use with `%`.\n'    
 'It is presented here in its own module for reuse:\n'    
 '\n'    
 '    from cs.x import X\n'    
 '    ...\n'    
 '    X("foo: x=%s, a=%r", x, a)\n'    
 '\n'    
 'It normally writes directly to `sys.stderr` but accepts an optional\n'    
 'keyword argument `file` to specify a different filelike object.\n'    
 '\n'    
 'The following globals are further tune its behaviour,\n'    
 'absent the `file=` parameter:\n'    
 '* `X_logger`: if not `None` then log a warning to that logger\n'    
 '* `X_via_tty`: if true then a pathname to which to append messages\n'    
 '* `X_discard`: if true then discard the message\n'    
 'Otherwise write the message to `sys.stderr`.\n'    
 '\n'    
 'If the environment variable `$CS_X_VIA_TTY` is empty,\n'    
 '`X_via_tty` will be false.\n'    
 'Otherwise,\n'    
 'if `$CS_X_VIA_TTY` has a nonempty value which is a full path\n'    
 'to an existing filesystem object (typically a tty)\n'    
 'then is will be used for `X_via_tty`,\n'    
 "otherwise `X_via_tty` will be set to `'/dev/tty'`.\n"    
 'This is handy for getting debugging out of test suites,\n'    
 'which often divert `sys.stderr`.\n'    
 '\n'    
 "`X_discard`'s default value is `not sys.stderr.isatty()`.\n"    
 '\n'    
 '## Function `X(msg, *args, **kw)`\n'    
 '\n'    
 'Unconditionally write the message `msg`.\n'    
 '\n'    
 'If there are positional arguments after `msg`,\n'    
 'format `msg` using %-expansion with those arguments.\n'    
 '\n'    
 'Keyword arguments:\n'    
 '* `file`: optional keyword argument specifying the output file.\n'    
 '* `colour`: optional text colour.\n'    
 '  If specified, surround the message with ANSI escape sequences\n'    
 '  to render the text in that colour.\n'    
 '\n'    
 'If `file` is not `None`, write to it unconditionally.\n'    
 'Otherwise, the following globals are consulted in order:\n'    
 '* `X_logger`: if not `None` then log a warning to that logger\n'    
 '* `X_via_tty`: if true then append the message to the path it contains\n'    
 '* `X_discard`: if true then discard the message\n'    
 'Otherwise write the message to `sys.stderr`.\n'    
 '\n'    
 '`X_logger` is `None` by default.\n'    
 '`X_via_tty` is initialised from the environment variable `$CS_X_VIA_TTY`.\n'    
 '`X_discard` is true unless `sys.stderr.isatty()` is true.\n'    
 '\n'    
 '## Function `Xtty(msg, *args, **kw)`\n'    
 '\n'    
 'Call `X()` with `X_via_tty` set to `True`.\n'    
 '\n'    
 '*Note*:\n'    
 'this is now obsoleted by the `$CS_X_VIA_TTY` environment variable.\n'    
 '\n'    
 'This supports using:\n'    
 '\n'    
 '    from cs.x import Xtty as X\n'    
 '\n'    
 'when hacking on tests without the tedious shuffle:\n'    
 '\n'    
 '    from cs.x import X\n'    
 '    import cs.x; cs.x.X_via_tty = True\n'    
 '\n'    
 'which I did _a lot_ to get timely debugging when fixing test failures.\n'    
 '\n'    
 '## Function `Y(msg, *a, **kw)`\n'    
 '\n'    
 'Wrapper for `X()` rendering in yellow.\n'    
 '\n'    
 '# Release Log\n'    
 '\n'    
 '\n'    
 '\n'    
 '*Release 20211208*:\n'    
 'X_via_tty: now accept full path to a tty in $CS_X_VIA_TTY to aid messaging '    
 'to other terminals.\n'    
 '\n'    
 '*Release 20210123*:\n'    
 'X: honour new $CS_X_COLOUR environment variable setting the default colour, '    
 'default uncoloured.\n'    
 '\n'    
 '*Release 20201227*:\n'    
 'New Y() which calls X(...,colour=yellow) - I now often go `from cs.x import '    
 'Y as X`.\n'    
 '\n'    
 '*Release 20201102*:\n'    
 '* Set X_via_tty if $CS_X_VIA_TTY.\n'    
 '* Put X() into builtins if $CS_X_BUILTIN.\n'    
 '\n'    
 '*Release 20181231*:\n'    
 '* X: trivial ANSI colour support via new `colour` keyword argument.\n'    
 '* New global X_discard, False unless sys.stderr.isatty.\n'    
 '\n'    
 '*Release 20180726*:\n'    
 'doco improvements\n'    
 '\n'    
 '*Release 20170902*:\n'    
 'Move X() into its own module, used for ad hoc debugging everywhere.\n'    
 '\n'    
 '*Release 20170707.3*:\n'    
 'tweak DISTINFO\n'    
 '\n'    
 '*Release 20170707.2*:\n'    
 'Doc tweak.\n'    
 '\n'    
 '*Release 20170707.1*:\n'    
 'Added README.\n'    
 '\n'    
 '*Release 20170707*:\n'    
 'Separate X() out into new module cs.x for cheap import.'),
  install_requires = ['cs.ansi_colour'],
  classifiers = ['Programming Language :: Python', 'Programming Language :: Python :: 2', 'Programming Language :: Python :: 3', 'Development Status :: 4 - Beta', 'Intended Audience :: Developers', 'Operating System :: OS Independent', 'Topic :: Software Development :: Libraries :: Python Modules', 'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)'],
  keywords = ['python2', 'python3'],
  license = 'GNU General Public License v3 or later (GPLv3+)',
  long_description_content_type = 'text/markdown',
  package_dir = {'': 'lib/python'},
  py_modules = ['cs.x'],
)



#!/usr/bin/python
#
# Just my X debugging function.
#   - Cameron Simpson <cs@cskk.id.au>
#

'''
X(), for low level debugging.

X() is my function for low level ad hoc debug messages.
It takes a message and optional format arguments for use with `%`.
It is presented here in its own module for reuse:

    from cs.x import X
    ...
    X("foo: x=%s, a=%r", x, a)

It normally writes directly to `sys.stderr` but accepts an optional
keyword argument `file` to specify a different filelike object.

The following globals are further tune its behaviour,
absent the `file=` parameter:
* `X_logger`: if not `None` then log a warning to that logger
* `X_via_tty`: if true then a pathname to which to append messages
* `X_discard`: if true then discard the message
Otherwise write the message to `sys.stderr`.

If the environment variable `$CS_X_VIA_TTY` is empty,
`X_via_tty` will be false.
Otherwise,
if `$CS_X_VIA_TTY` has a nonempty value which is a full path
to an existing filesystem object (typically a tty)
then is will be used for `X_via_tty`,
otherwise `X_via_tty` will be set to `'/dev/tty'`.
This is handy for getting debugging out of test suites,
which often divert `sys.stderr`.

`X_discard`'s default value is `not sys.stderr.isatty()`.
'''

from __future__ import print_function
import os
import os.path
import sys
from cs.ansi_colour import colourise

__version__ = '20211208'

DISTINFO = {
    'keywords': ["python2", "python3"],
    'classifiers': [
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
    ],
    'install_requires': ['cs.ansi_colour'],
}

# discard output? the default if sys.stderr is not a tty
try:
  isatty = sys.stderr.isatty
except AttributeError:
  X_discard = True
else:
  X_discard = not isatty()
# set to a logger to log as a warning
X_logger = None
# colouring
X_default_colour = os.environ.get('CS_X_COLOUR')

def X(msg, *args, **kw):
  ''' Unconditionally write the message `msg`.

      If there are positional arguments after `msg`,
      format `msg` using %-expansion with those arguments.

      Keyword arguments:
      * `file`: optional keyword argument specifying the output file.
      * `colour`: optional text colour.
        If specified, surround the message with ANSI escape sequences
        to render the text in that colour.

      If `file` is not `None`, write to it unconditionally.
      Otherwise, the following globals are consulted in order:
      * `X_logger`: if not `None` then log a warning to that logger
      * `X_via_tty`: if true then append the message to the path it contains
      * `X_discard`: if true then discard the message
      Otherwise write the message to `sys.stderr`.

      `X_logger` is `None` by default.
      `X_via_tty` is initialised from the environment variable `$CS_X_VIA_TTY`.
      `X_discard` is true unless `sys.stderr.isatty()` is true.
  '''
  fp = kw.pop('file', None)
  colour = kw.pop('colour', X_default_colour)
  if kw:
    raise ValueError("unexpected keyword arguments: %r" % (kw,))
  msg = str(msg)
  if args:
    msg = msg % args
  if colour:
    msg = colourise(msg, colour=colour)
  if fp is None:
    if X_logger:
      # NB: ignores any kwargs
      X_logger.warning(msg)
      return
    if X_via_tty:
      # NB: ignores any kwargs
      try:
        with open(X_via_tty, 'a') as f:
          f.write(msg)
          f.write('\n')
      except (IOError, OSError) as e:
        X("X: cannot append to %r: %s", X_via_tty, e, file=sys.stderr)
        X(msg, file=sys.stderr)
      return
    if X_discard:
      return
    fp = sys.stderr
  print(msg, file=fp)

# init X_via_tty (after X() because we use X() for messaging
env_via_tty = os.environ.get('CS_X_VIA_TTY', '')
if not env_via_tty:
  X_via_tty = False
else:
  X_via_tty = '/dev/tty'
  if env_via_tty.startswith('/'):
    try:
      xfd = os.open(env_via_tty, os.O_RDONLY)
    except OSError as e:
      X("%s: open(%r): %s, using %r", __file__, env_via_tty, e, X_via_tty)
    else:
      if not os.isatty(xfd):
        X(
            "%s: open(%r): not a tty, using %r", __file__, env_via_tty,
            X_via_tty
        )
      else:
        X_via_tty = env_via_tty
      os.close(xfd)

if os.environ.get('CS_X_BUILTIN', ''):
  try:
    import builtins
  except ImportError:
    pass
  else:
    builtins.X = X

def Xtty(msg, *args, **kw):
  ''' Call `X()` with `X_via_tty` set to `True`.

      *Note*:
      this is now obsoleted by the `$CS_X_VIA_TTY` environment variable.

      This supports using:

          from cs.x import Xtty as X

      when hacking on tests without the tedious shuffle:

          from cs.x import X
          import cs.x; cs.x.X_via_tty = True

      which I did _a lot_ to get timely debugging when fixing test failures.
  '''
  global X_via_tty  # pylint: disable=global-statement
  old = X_via_tty
  X_via_tty = True
  X(msg, *args, **kw)
  X_via_tty = old

def Y(msg, *a, **kw):
  ''' Wrapper for `X()` rendering in yellow.
  '''
  X(msg, *a, colour='yellow', **kw)
